﻿using System.Threading;
using ZMQ;
using System;
using System.Threading.Tasks;
using SimpleBroadcast;
using System.Text;

namespace SimpleBroadcast
{
    public class BrokerService
    {

        private Socket multiSub;
        private Socket multiPub;

        private int sendCount = 0;
        private int multiCount = 0;

        private CancellationTokenSource _cancelationToken;
        private Context _context;


        public void Start()
        {
            _context = new Context(2);
            
            _cancelationToken = new CancellationTokenSource();

            PollingLoop();
        }
        
        public void Stop()
        {
           
            _cancelationToken.Cancel();

        }

        private void PollingLoop()
        {
            InitSockets();

            Task multiSubTask = Task.Factory.StartNew(() =>
            {
                while (!_cancelationToken.IsCancellationRequested)
                {
                    MultiSubHandler(multiSub, IOMultiPlex.POLLIN);
                }
            }, _cancelationToken.Token);

            Task localSubTask = Task.Factory.StartNew(() =>
            {
                while (!_cancelationToken.IsCancellationRequested)
                {
                    multiPub.Send("Message", Encoding.UTF8);

                    sendCount++;
                    if (sendCount % 100000 == 0)
                    {
                        Console.WriteLine("Send {0} messages", sendCount);
                        //Thread.Sleep(10000);
                    }
                }
            }, _cancelationToken.Token);
        }

        private void InitSockets()
        {
          

            multiSub = _context.Socket(SocketType.SUB);
            multiSub.HWM = 10000;
            multiSub.Rate = 1000000;
            multiSub.RecoveryIvlMsec = 300;
            multiSub.Linger = 1000;

            // disable receiving messages from itself
            multiSub.SetSockOpt(SocketOpt.MCAST_LOOP, 0L);
            multiSub.Subscribe("", Encoding.UTF8); 

            //multiSub.PollInHandler += MultiSubHandler;
            multiSub.PollErrHandler += new PollHandler(multiSub_PollErrHandler);

            multiSub.Connect("epgm://;224.0.0.20:1000");
 
            multiPub = _context.Socket(SocketType.PUB);
            multiPub.HWM = 10000;
            multiPub.Rate = 1000000;
            multiPub.RecoveryIvlMsec = 300;
            multiPub.Linger = 1000;
            // disable receiving messages from itself
            multiPub.SetSockOpt(SocketOpt.MCAST_LOOP, 0L);
            multiPub.Connect("epgm://;224.0.0.20:1000");
            
        }

        void multiSub_PollErrHandler(Socket socket, IOMultiPlex revents)
        {
            Console.WriteLine("Error multi sub");
        }

       

        private void MultiSubHandler(Socket socket, IOMultiPlex revents)
        {

            byte[] message = null;
            message = socket.Recv();
            
            if (message != null)
            {

               
                while (socket.RcvMore)
                {
                    message = socket.Recv();
                }
                
                multiCount++;
                if ((multiCount % 1000) == 0)
                {
                    Console.WriteLine("Received multi {0}", multiCount);
                }
            }
            else 
            {
                Console.WriteLine("Smth is wrong");

            }
        }

       
    }
}
